@@ -5,7 +5,7 @@ use 5.006;
 use strict;
 use warnings;
 
-our $VERSION = '0.14_05';
+our $VERSION = '0.15_01';
 $VERSION =~ tr/_//d;
 
 BEGIN {
@@ -18,12 +18,9 @@ sub _warn { warn &_longmess }
 sub _die { die ref $_[0] ? @_ : &_longmess }
 
 sub _longmess {
-  if ($_[-1] =~ /\n\z/) {
-    my $arg = pop @_;
-    $arg =~ s/(.*) at [^\n]*? line [0-9]+\.?\n$/$1/s;
-    push @_, $arg;
-  }
-  &Carp::longmess;
+  my $mess = &Carp::longmess;
+  $mess =~ s/( at .*?\n)\1/$1/s;    # Suppress duplicate tracebacks
+  $mess;
 }
 
 my @HOOKS = qw(__DIE__ __WARN__);
@@ -32,7 +29,7 @@ my %OLD_SIG;
 sub import {
   my $class = shift;
   return if $OLD_SIG{$class};
-  @{ $OLD_SIG{$class} }{@HOOKS} = @SIG{@HOOKS};
+  @{ $OLD_SIG{$class} }{ @HOOKS, 'Verbose' } = (@SIG{@HOOKS}, $Carp::Verbose);
 
   @SIG{@HOOKS} = ($class->can('_die'), $class->can('_warn'));
   $Carp::Verbose = 'verbose';    # makes carp() cluck and croak() confess
@@ -41,8 +38,7 @@ sub import {
 sub unimport {
   my $class = shift;
   return unless $OLD_SIG{$class};
-  @SIG{@HOOKS} = @{ delete $OLD_SIG{$class} }{@HOOKS};
-  undef $Carp::Verbose;
+  (@SIG{@HOOKS}, $Carp::Verbose) = @{ delete $OLD_SIG{$class} }{ @HOOKS, 'Verbose' };
 }
 
 1;
@@ -86,7 +82,7 @@ looks:
 In the implementation, the L<Carp> module does
 the heavy work, through C<longmess()>. The
 actual implementation sets the signal hooks
-C<$SIG{__WARN__}> and C<$SIG{__DIE__}> to
+L<$SIG{__WARN__}|perlvar/%SIG> and L<$SIG{__DIE__}|perlvar/%SIG> to
 emit the stack backtraces.
 
 Also, all uses of C<carp> and C<croak> are made verbose,
@@ -143,6 +139,8 @@ L<Carp::Source::Always>
 
 L<Devel::Confess>
 
+L<Carp::Always::SyntaxHighlightSource> and L<Carp::Always::DieOnly>
+
 =head1 BUGS
 
 =over 4
@@ -150,8 +148,7 @@ L<Devel::Confess>
 =item *
 
 This module does not play well with other modules which fusses
-around with C<warn>, C<die>, C<$SIG{__WARN__}>,
-C<$SIG{__DIE__}>.
+around with C<warn>, C<die>, C<$SIG{__WARN__}>, C<$SIG{__DIE__}>.
 
 =item *
 
